@@ -1,4 +1,4 @@
-import sys, time
+import sys, time, os, shutil
 import numpy as np
 import dask.array as da
 import SimpleITK as sitk
@@ -11,6 +11,7 @@ from CircuitSeeker.transform import compose_displacement_vector_fields
 from CircuitSeeker.quality import jaccard_filter
 from CircuitSeeker.metrics import patch_mutual_information
 from scipy.spatial.transform import Rotation
+import zarr
 
 
 def random_affine_search(
@@ -880,6 +881,7 @@ def distributed_piecewise_alignment_pipeline(
     deform_kwargs={},
     cluster=None,
     cluster_kwargs={},
+    temporary_directory=None,
     **kwargs,
 ):
     """
@@ -991,7 +993,7 @@ def distributed_piecewise_alignment_pipeline(
     # compute block size and overlaps
     blocksize = np.array(fix.shape).astype(np.float32) / nblocks
     blocksize = np.ceil(blocksize).astype(np.int16)
-    overlaps = tuple(np.round(blocksize * overlap).astype(np.int16))
+    overlaps = np.round(blocksize * overlap).astype(np.int16)
 
     # pad the ends to fill in the last blocks
     # blocks must all be exact for stitch to work correctly
@@ -999,43 +1001,39 @@ def distributed_piecewise_alignment_pipeline(
         else (0, 0) for x, y in zip(fix.shape, blocksize)]
     fix_p = np.pad(fix, pads)
     mov_p = np.pad(mov, pads)
-
-    # pad masks if necessary
     if fix_mask is not None:
         fm_p = np.pad(fix_mask, pads)
     if mov_mask is not None:
         mm_p = np.pad(mov_mask, pads)
 
-    # CONSTRUCT DASK ARRAY VERSION OF OBJECTS
-    # fix
-    fix_future = cluster.client.scatter(fix_p)
-    fix_da = da.from_delayed(
-        fix_future, shape=fix_p.shape, dtype=fix_p.dtype
-    ).rechunk(tuple(blocksize))
+    # ensure temporary directory exists
+    if temporary_directory is None:
+        temporary_directory = os.getcwd()
+    temporary_directory += '/distributed_alignment_temp'
+    os.makedirs(temporary_directory)
 
-    # mov
-    mov_future = cluster.client.scatter(mov_p)
-    mov_da = da.from_delayed(
-        mov_future, shape=mov_p.shape, dtype=mov_p.dtype
-    ).rechunk(tuple(blocksize))
+    # define zarr paths
+    fix_zarr_path = temporary_directory + '/fix.zarr'
+    mov_zarr_path = temporary_directory + '/mov.zarr'
+    fix_mask_zarr_path = temporary_directory + '/fix_mask.zarr'
+    mov_mask_zarr_path = temporary_directory + '/mov_mask.zarr'
 
-    # fix mask
+    # create zarr files
+    zarr_blocks = (128,)*len(fix_p.shape)
+    ut.numpy_to_zarr(fix_p, zarr_blocks, fix_zarr_path)
+    ut.numpy_to_zarr(mov_p, zarr_blocks, mov_zarr_path)
     if fix_mask is not None:
-        fm_future = cluster.client.scatter(fm_p)
-        fm_da = da.from_delayed(
-            fm_future, shape=fm_p.shape, dtype=fm_p.dtype
-        ).rechunk(tuple(blocksize))
-    else:
-        fm_da = da.from_array([None,], chunks=(1,))
-
-    # mov mask
+        ut.numpy_to_zarr(fm_p, zarr_blocks, fix_mask_zarr_path)
     if mov_mask is not None:
-        mm_future = cluster.client.scatter(mm_p)
-        mm_da = da.from_delayed(
-            mm_future, shape=mm_p.shape, dtype=mm_p.dtype
-        ).rechunk(tuple(blocksize))
-    else:
-        mm_da = da.from_array([None,], chunks=(1,))
+        ut.numpy_to_zarr(mm_p, zarr_blocks, mov_mask_zarr_path)
+
+    # construct start/stop indices for each block (includes overlap)
+    slc = tuple(slice(None, x, y) for x, y in zip(fix_p.shape, blocksize))
+    starts = np.moveaxis(np.array(np.mgrid[slc]), 0, -1) - overlaps
+    stops = starts + blocksize + 2 * overlaps
+    starts = np.maximum(0, starts)  # no boundary overlap
+    indices = np.concatenate((starts, stops), axis=-1).astype(int)
+    indices_da = da.from_array(indices, chunks=(1,)*fix_p.ndim + (2*fix_p.ndim,))
 
     # establish all keyword arguments
     random_kwargs = {**kwargs, **random_kwargs}
@@ -1044,16 +1042,26 @@ def distributed_piecewise_alignment_pipeline(
     deform_kwargs = {**kwargs, **deform_kwargs}
 
     # closure for alignment pipeline
-    def align_single_block(fix, mov, fm, mm, block_info=None):
-        
-        # check for masks
-        if fm.shape != fix.shape: fm = None
-        if mm.shape != mov.shape: mm = None
+    def align_single_block(coords, block_info=None):
+
+        # squeeze the coords and convert to tuple of slices
+        coords = coords.squeeze()
+        d = len(coords) // 2
+        coords = tuple(slice(x, y) for x, y in zip(coords[:d], coords[d:]))
+
+        # read the chunks
+        fix = zarr.open(fix_zarr_path, 'r')[coords]
+        mov = zarr.open(mov_zarr_path, 'r')[coords]
+        fix_mask, mov_mask = None, None
+        if os.path.isdir(fix_mask_zarr_path):
+            fix_mask = zarr.open(fix_mask_zarr_path, 'r')[coords]
+        if os.path.isdir(mov_mask_zarr_path):
+            mov_mask = zarr.open(mov_mask_zarr_path, 'r')[coords]
 
         # run alignment pipeline
         transform = alignment_pipeline(
             fix, mov, fix_spacing, mov_spacing, steps,
-            fix_mask=fm, mov_mask=mm,
+            fix_mask=fix_mask, mov_mask=mov_mask,
             random_kwargs=random_kwargs,
             rigid_kwargs=rigid_kwargs,
             affine_kwargs=affine_kwargs,
@@ -1111,27 +1119,13 @@ def distributed_piecewise_alignment_pipeline(
         return result
     # END CLOSURE
 
-    # determine overlaps list
-    overlaps_list = [overlaps, overlaps]
-    if fix_mask is not None:
-        overlaps_list.append(overlaps)
-    else:
-        overlaps_list.append(0)
-    if mov_mask is not None:
-        overlaps_list.append(overlaps)
-    else:
-        overlaps_list.append(0)
-
     # align all chunks
-    fields = da.map_overlap(
-        align_single_block,
-        fix_da, mov_da, fm_da, mm_da,
-        depth=overlaps_list,
+    fields = da.map_blocks(
+        align_single_block, indices_da,
         dtype=object,
-        boundary='none',
-        trim=False,
-        align_arrays=False,
+        drop_axis=[len(indices_da.shape)-1],
         chunks=[1,1,1],
+        meta=np.array((), dtype=object)
     ).compute()
 
     # reconstruct single transform
@@ -1145,6 +1139,9 @@ def distributed_piecewise_alignment_pipeline(
         xr, yr, zr = fields[i, j, k].shape[:3]
         transform[x:x+xr, y:y+yr, z:z+zr] += fields[i, j, k][...]
 
+    # remove temporary files
+    shutil.rmtree(temporary_directory)
+
     # crop back to original shape and return
     x, y, z = fix.shape
     return transform[:x, :y, :z]
