@@ -4,6 +4,8 @@ import functools
 import ClusterWrap
 import dask
 from scipy.ndimage import zoom
+import zarr
+from numcodecs import Blosc
 
 
 def skip_sample(image, spacing, ss_spacing):
@@ -155,3 +157,24 @@ def scatter_dask_array(cluster, array):
     else:
         return array
 
+
+def numpy_to_zarr(array, chunks, path):
+    """
+    """
+
+    if not isinstance(array, zarr.Array):
+        compressor = Blosc(
+            cname='zstd', clevel=4, shuffle=Blosc.BITSHUFFLE,
+        )
+        zarr_disk = zarr.open(
+            path, 'w',
+            shape=array.shape,
+            chunks=chunks,
+            dtype=array.dtype,
+            compressor=compressor,
+        )
+        zarr_disk[...] = array
+        return zarr_disk
+    else:
+        return array
+
